/* ------------------------------------------------------------------ */
/* NSF Project: DUE 9653244                                           */
/* Geometric Computing in Undergraduate Computer Science Curricula    */
/*                                                                    */
/* Principal Investigators: Dr. Ching-Kuang Shene,  Dr. John Lowther  */
/*                                                                    */
/* Developed by Geometric Computing Team Member: Yuan Zhao            */
/*                                                                    */
/* Version 0.01, 06/18/97                                             */
/*                                                                    */
/* Copyright (c) Yuan Zhao, Ching-Kuang Shene, and John Lowther, 1997,*/
/* subject to NSF Grant General Condidtions (GC-1)                    */
/*                                                                    */
/* ALL RIGHTS RESERVED                                                */
/* Permission to use, copy, modify, and distribute this software for  */
/* any purpose and without fee is hereby granted, provided that the   */ 
/* above copyright notice appear in all copies and that both the      */
/* copyright notice and this permission notice appear in supporting   */
/* documentation.                                                     */
/*                                                                    */
/* THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"  */
/* AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,   */
/* INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR   */
/* FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL MICHIGAN      */
/* TECHNOLOGICAL UNIVERSITY NOR NSF BE LIABLE TO YOU OR ANYONE ELSE   */
/* FOR ANY DIRECT, SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL     */
/* DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT  */
/* LIMITATION, LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR    */
/* THE CLAIMS OF THIRD PARTIES, WHETHER WE HAVE BEEN ADVISED OF THE   */
/* POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON ANY THEORY OF      */
/* LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE POSSESSION,    */
/* USE OR PERFORMANCE OF THIS SOFTWARE.                               */
/* ------------------------------------------------------------------ */

/* util.c
 *	Some useful utilities
 */

#include <stdio.h>
#include <GL/glut.h>

#include "colors.h"
#include "util.h"
#include "defines.h"

void gmtuButtonEffect(int x1, int y1, int x2, int y2)
{
    glColor4f(WHITE);
    glLineWidth(3);
    glBegin(GL_LINE_STRIP);
	glVertex2i(x1, y1);
	glVertex2i(x1, y2);
	glVertex2i(x2, y2);
    glEnd();

    glColor4f(DIMGREY);
    glLineWidth(2);
    glBegin(GL_LINE_STRIP);
	glVertex2i(x1, y1);
	glVertex2i(x2, y1);
	glVertex2i(x2, y2);
    glEnd();
}

void gmtuButtonInverseDraw(gmtu_Button * in_b) {
    GLfloat color[4];

    glGetFloatv(GL_CURRENT_COLOR, color);
    glColor4f(GREY);
    glBegin(GL_POLYGON);
	glVertex2i(in_b->x1 - 3, in_b->y1 - 3);
	glVertex2i(in_b->x2 + 3, in_b->y1 - 3);
	glVertex2i(in_b->x2 + 3, in_b->y2 + 4);
	glVertex2i(in_b->x1 - 3, in_b->y2 + 4);
    glEnd();
    glColor4fv(color);
    gmtuButtonDraw(in_b);
    /* glLineWidth(3);
    glBegin(GL_POLYGON);
	glVertex2i(in_b->x1 - 2, in_b->y1 -2);
	glVertex2i(in_b->x1 - 2, in_b->y1 -2);
    glEnd(); */
}

/* Use BITMAP_9_BY_15 Font */
void gmtuButtonDraw(gmtu_Button *in_b)
{
    int i, ybase;

    ybase = (in_b->y2 - in_b->y1) < 0 ? (in_b->y1 - in_b->y2) 
					: (in_b->y2 - in_b->y1);
    ybase = (ybase - GMTU_FONT_HEIGHT) < 0 ? 0 : (ybase - GMTU_FONT_HEIGHT);
    ybase = ybase/2 + in_b->y1 + 2;
    glRasterPos2i(in_b->x1 + 3, ybase);
    for(i = 0; in_b->str[i] != '\0'; i++){
	/* glutBitmapCharacter(GLUT_BITMAP_9_BY_15, string[i]); */
	/* glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_10, string[i]); */
	glutBitmapCharacter(in_b->font, in_b->str[i]);
    }
    gmtuButtonEffect(in_b->x1, in_b->y1, in_b->x2, in_b->y2);
}

int gmtuButtonIn(gmtu_Button *in_b, int x, int y)
{
    if(in_b->x1 <= x && x <= in_b->x2 && in_b->y1 <= y && y <= in_b->y2)
	return TRUE;
    else
	return FALSE;
}

void gmtuButton(int x1, int y1, int x2, int y2, char *string, void *font)
{
    int i, ybase;

    ybase = (y2 - y1) < 0 ? (y1 - y2) : (y2 - y1);
    ybase = (ybase - GMTU_FONT_HEIGHT) < 0 ? 0 : (ybase - GMTU_FONT_HEIGHT);
    ybase = ybase/2 + y1 + 2;
    glRasterPos2i(x1 + 3, ybase);
    for(i = 0; string[i] != '\0'; i++){
	/* glutBitmapCharacter(GLUT_BITMAP_9_BY_15, string[i]); */
	/* glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_10, string[i]); */
	glutBitmapCharacter(font, string[i]);
    }
    gmtuButtonEffect(x1, y1, x2, y2);
}

void gmtuDrawPara(char *string, int x, int y, int gap, void *font)
{
    int i, j;

    for(i = 0, j = 0; string[i] != '\0'; i++, j++) {
    	glRasterPos2i(x, y - j * gap);
	for(; string[i] != '\0'; i++) {
	    if(string[i] == '\n') break;
	    glutBitmapCharacter(font, string[i]);
	}

	if(string[i] == '\0') break;	/* for safety reasons */
    }
}

